#include "SelectObject.h"
#include <ngl/ShaderManager.h>
#include <ngl/VBOPrimitives.h>

// set the initial colour to 0
unsigned char SelectObject::s_gColourID[3] = {0, 0, 0};

SelectObject::SelectObject(ngl::Vector pos)
{

// set the initial values
m_pos=pos;
m_active=false;
// generate an automatic colour id value for selection
// grab the current active colour id values
m_colourID[0] = s_gColourID[0];
m_colourID[1] = s_gColourID[1];
m_colourID[2] = s_gColourID[2];
// now we need to increment the values for the next one
//1 add to the red chan value
++s_gColourID[0];
// if we are at full value
if(s_gColourID[0] == 255)
	{
	// set red channel to 0
	s_gColourID[0] = 0;
	// inc blue channel
	++s_gColourID[1];
	// if full go to next
	if(s_gColourID[1] == 255)
		{
		 s_gColourID[1] = 0;
		 ++s_gColourID[2];
		}
	}

}

void SelectObject::checkSelectionColour(const unsigned char col[3])
{
// see if the colour passed in is the same as our objects one
if(m_colourID[0] == col[0] && m_colourID[1] == col[1] && m_colourID[2] == col[2])
{
	m_active^=true;
}

}

void SelectObject::draw(
													bool _selection,
													const std::string &_shaderName,
													ngl::TransformStack &_transformStack
												)
{
// grab the VBO instance
ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
ngl::ShaderManager *shader=ngl::ShaderManager::instance();

if(_selection == true)
{
	shader->setShaderParam4f(_shaderName,"Colour",m_colourID[0]/255.0f, m_colourID[1]/255.0f,m_colourID[2]/255.0f,1);
}
_transformStack.pushTransform();
	{
		_transformStack.getCurrentTransform().setPosition(m_pos);
		shader->setShaderParamFromMatrix(_shaderName,"ModelMatrix",_transformStack.getCurrAndGlobal().getMatrix());
		if(m_active == true)
		{
			// draw a teapot
			prim->draw("teapot");
		}
		else
		{
			// draw a cube
			prim->draw("cube");
		}
	} // and before a pop
  _transformStack.popTransform();

}
